@@ -49,7 +49,12 @@ import shutil
 import sys
 import tempfile
 import tokenize
+import unittest
+import warnings
+from importlib import reload
 from importlib.abc import Loader, MetaPathFinder
+from importlib.machinery import ModuleSpec
+from importlib.util import spec_from_file_location, module_from_spec
 from types import ModuleType, TracebackType, FunctionType
 from typing import (
     Any,
@@ -66,10 +71,13 @@ from typing import (
     ItemsView,
     Sequence,
 )
-import unittest
-import warnings
 from unittest import TestSuite
 
+from pyfakefs import fake_filesystem, fake_io, fake_os, fake_open, fake_path, fake_file
+from pyfakefs import fake_filesystem_shutil
+from pyfakefs import fake_legacy_modules
+from pyfakefs import fake_pathlib
+from pyfakefs import mox3_stubout
 from pyfakefs.fake_filesystem import (
     set_uid,
     set_gid,
@@ -77,23 +85,50 @@ from pyfakefs.fake_filesystem import (
     PatchMode,
     FakeFilesystem,
 )
+from pyfakefs.fake_os import use_original_os
 from pyfakefs.helpers import IS_PYPY
-from pyfakefs.mox3_stubout import StubOutForTesting
-
-from importlib.machinery import ModuleSpec
-from importlib import reload
-
-from pyfakefs import fake_filesystem, fake_io, fake_os, fake_open, fake_path, fake_file
-from pyfakefs import fake_legacy_modules
-from pyfakefs import fake_filesystem_shutil
-from pyfakefs import fake_pathlib
-from pyfakefs import mox3_stubout
 from pyfakefs.legacy_packages import pathlib2, scandir
+from pyfakefs.mox3_stubout import StubOutForTesting
 
 OS_MODULE = "nt" if sys.platform == "win32" else "posix"
 PATH_MODULE = "ntpath" if sys.platform == "win32" else "posixpath"
 
 
+class TempfilePatcher:
+    """Handles tempfile patching for Posix systems."""
+
+    def __init__(self):
+        self.tempfile_cleanup = None
+
+    def start_patching(self):
+        if self.tempfile_cleanup is not None:
+            return
+        if sys.version_info >= (3, 12):
+
+            def cleanup(self_, windows=(os.name == "nt"), unlink=None):
+                self.tempfile_cleanup(self_, windows, unlink or os.unlink)
+
+            self.tempfile_cleanup = tempfile._TemporaryFileCloser.cleanup  # type: ignore[module-attr]
+            tempfile._TemporaryFileCloser.cleanup = cleanup  # type: ignore[module-attr]
+        elif sys.platform != "win32":
+
+            def close(self_, unlink=None):
+                self.tempfile_cleanup(self_, unlink or os.unlink)
+
+            self.tempfile_cleanup = tempfile._TemporaryFileCloser.close  # type: ignore[module-attr]
+            tempfile._TemporaryFileCloser.close = close  # type: ignore[module-attr]
+
+    def stop_patching(self):
+        if self.tempfile_cleanup is None:
+            return
+        if sys.version_info < (3, 12):
+            tempfile._TemporaryFileCloser.close = self.tempfile_cleanup  # type: ignore[module-attr]
+        else:
+            tempfile._TemporaryFileCloser.cleanup = self.tempfile_cleanup  # type: ignore[module-attr]
+        # reset the cached tempdir in tempfile
+        tempfile.tempdir = None
+
+
 def patchfs(
     _func: Optional[Callable] = None,
     *,
@@ -479,6 +514,13 @@ class Patcher:
         SKIPMODULES.add(posixpath)
         SKIPMODULES.add(fcntl)
 
+    # a list of modules detected at run-time
+    # each tool defines one or more module name prefixes for modules to be skipped
+    RUNTIME_SKIPMODULES = {
+        "pydevd": ["_pydevd_", "pydevd", "_pydev_"],  # Python debugger (PyCharm/VSCode)
+        "_jb_runner_tools": ["_jb_"],  # JetBrains tools
+    }
+
     # caches all modules that do not have file system modules or function
     # to speed up _find_modules
     CACHED_MODULES: Set[ModuleType] = set()
@@ -569,17 +611,20 @@ class Patcher:
             set_uid(1)
             set_gid(1)
 
-        self._skip_names = self.SKIPNAMES.copy()
+        self.skip_names = self.SKIPNAMES.copy()
         # save the original open function for use in pytest plugin
         self.original_open = open
         self.patch_open_code = patch_open_code
+        self.linecache_updatecache = None
+        self.linecache_checkcache = None
+        self.tempfile_patcher = TempfilePatcher()
 
         if additional_skip_names is not None:
             skip_names = [
                 cast(ModuleType, m).__name__ if inspect.ismodule(m) else cast(str, m)
                 for m in additional_skip_names
             ]
-            self._skip_names.update(skip_names)
+            self.skip_names.update(skip_names)
 
         self._fake_module_classes: Dict[str, Any] = {}
         self._unfaked_module_classes: Dict[str, Any] = {}
@@ -587,12 +632,7 @@ class Patcher:
         self._init_fake_module_classes()
 
         # reload tempfile under posix to patch default argument
-        self.modules_to_reload: List[ModuleType] = (
-            [] if sys.platform == "win32" else [tempfile]
-        )
-        if sys.version_info >= (3, 13):
-            # need to reload glob which holds references to os functions
-            self.modules_to_reload.append(glob)
+        self.modules_to_reload: List[ModuleType] = []
         if modules_to_reload is not None:
             self.modules_to_reload.extend(modules_to_reload)
         self.patch_default_args = patch_default_args
@@ -625,8 +665,8 @@ class Patcher:
             if patched_module_names != self.PATCHED_MODULE_NAMES:
                 self.__class__.PATCHED_MODULE_NAMES = patched_module_names
                 clear_cache = True
-            if self._skip_names != self.ADDITIONAL_SKIP_NAMES:
-                self.__class__.ADDITIONAL_SKIP_NAMES = self._skip_names
+            if self.skip_names != self.ADDITIONAL_SKIP_NAMES:
+                self.__class__.ADDITIONAL_SKIP_NAMES = self.skip_names
                 clear_cache = True
             if patch_default_args != self.PATCH_DEFAULT_ARGS:
                 self.__class__.PATCH_DEFAULT_ARGS = patch_default_args
@@ -649,6 +689,18 @@ class Patcher:
         self._patching = False
         self._paused = False
 
+    def checkcache(self, filename=None):
+        """Calls the original linecache.checkcache making sure no fake OS calls
+        are used."""
+        with use_original_os():
+            return self.linecache_checkcache(filename)
+
+    def updatecache(self, filename, module_globals=None):
+        """Calls the original linecache.updatecache making sure no fake OS calls
+        are used."""
+        with use_original_os():
+            return self.linecache_updatecache(filename, module_globals)
+
     @classmethod
     def clear_fs_cache(cls) -> None:
         """Clear the module cache."""
@@ -860,7 +912,7 @@ class Patcher:
                         pass
                 continue
             skipped = module in self.SKIPMODULES or any(
-                [sn.startswith(module.__name__) for sn in self._skip_names]
+                [sn.startswith(module.__name__) for sn in self.skip_names]
             )
             module_items = module.__dict__.copy().items()
 
@@ -907,7 +959,7 @@ class Patcher:
         for name in self._fake_module_classes:
             self.fake_modules[name] = self._fake_module_classes[name](self.fs)
             if hasattr(self.fake_modules[name], "skip_names"):
-                self.fake_modules[name].skip_names = self._skip_names
+                self.fake_modules[name].skip_names = self.skip_names
         self.fake_modules[PATH_MODULE] = self.fake_modules["os"].path
         for name in self._unfaked_module_classes:
             self.unfaked_modules[name] = self._unfaked_module_classes[name]()
@@ -957,9 +1009,20 @@ class Patcher:
             self._patching = True
             self._paused = False
 
+            if sys.version_info >= (3, 13):
+                # in linecache, 'os' is now imported locally, which involves the
+                # dynamic patcher, therefore we patch the affected functions
+                self.linecache_updatecache = linecache.updatecache
+                linecache.updatecache = self.updatecache
+                self.linecache_checkcache = linecache.checkcache
+                linecache.checkcache = self.checkcache
+
+            self.tempfile_patcher.start_patching()
+
             self.patch_modules()
             self.patch_functions()
             self.patch_defaults()
+            self._set_glob_os_functions()
 
             self._dyn_patcher = DynamicPatcher(self)
             sys.meta_path.insert(0, self._dyn_patcher)
@@ -970,6 +1033,13 @@ class Patcher:
                 self._dyn_patcher.cleanup()
                 sys.meta_path.pop(0)
 
+    def _set_glob_os_functions(self):
+        # make sure the os functions cached in glob are patched
+        if sys.version_info >= (3, 13):
+            globber = glob._StringGlobber  # type: ignore[module-attr]
+            globber.lstat = staticmethod(os.lstat)
+            globber.scandir = staticmethod(os.scandir)
+
     def patch_functions(self) -> None:
         assert self._stubs is not None
         for (name, ft_name, ft_mod), modules in self.FS_FUNCTIONS.items():
@@ -982,10 +1052,26 @@ class Patcher:
                 self._stubs.smart_set(module, name, attr)
 
     def patch_modules(self) -> None:
+        skip_prefix_list = []
+        for rt_skip_module, prefixes in self.RUNTIME_SKIPMODULES.items():
+            if rt_skip_module in sys.modules:
+                skip_prefix_list.extend(prefixes)
+        skip_prefixes = tuple(skip_prefix_list)
+
         assert self._stubs is not None
         for name, modules in self.FS_MODULES.items():
             for module, attr in modules:
-                self._stubs.smart_set(module, name, self.fake_modules[attr])
+                try:
+                    if not skip_prefixes or not module.__name__.startswith(
+                        skip_prefixes
+                    ):
+                        self._stubs.smart_set(module, name, self.fake_modules[attr])
+                    elif attr in self.unfaked_modules:
+                        self._stubs.smart_set(module, name, self.unfaked_modules[attr])
+                except Exception:
+                    # handle the rare case that a module has no __name__
+                    pass
+
         for name, modules in self.SKIPPED_FS_MODULES.items():
             for module, attr in modules:
                 if attr in self.unfaked_modules:
@@ -1047,6 +1133,11 @@ class Patcher:
             if self.use_dynamic_patch and self._dyn_patcher:
                 self._dyn_patcher.cleanup()
                 sys.meta_path.pop(0)
+            self.tempfile_patcher.stop_patching()
+            if self.linecache_updatecache is not None:
+                linecache.updatecache = self.linecache_updatecache
+                linecache.checkcache = self.linecache_checkcache
+            self._set_glob_os_functions()
 
     @property
     def is_patching(self):
@@ -1156,7 +1247,7 @@ class DynamicPatcher(MetaPathFinder, Loader):
                 del sys.modules[name]
 
     def needs_patch(self, name: str) -> bool:
-        """Check if the module with the given name shall be replaced."""
+        """Checks if the module with the given name shall be replaced."""
         if name not in self.modules:
             self._loaded_module_names.add(name)
             return False
@@ -1164,6 +1255,24 @@ class DynamicPatcher(MetaPathFinder, Loader):
             return False
         return True
 
+    def fake_module_path(self, name: str) -> str:
+        """Checks if the module with the given name is a module existing in the fake
+        filesystem and returns its path in this case.
+        """
+        fs = self._patcher.fs
+        # we assume that the module name is the absolute module path
+        if fs is not None:
+            base_path = name.replace(".", fs.path_separator)
+            for path in sys.path:
+                module_path = fs.joinpaths(path, base_path)
+                py_module_path = module_path + ".py"
+                if fs.exists(py_module_path):
+                    return fs.absnormpath(py_module_path)
+                init_path = fs.joinpaths(module_path, "__init__.py")
+                if fs.exists(init_path):
+                    return fs.absnormpath(init_path)
+        return ""
+
     def find_spec(
         self,
         fullname: str,
@@ -1173,6 +1282,15 @@ class DynamicPatcher(MetaPathFinder, Loader):
         """Module finder."""
         if self.needs_patch(fullname):
             return ModuleSpec(fullname, self)
+        if self._patcher.patch_open_code != PatchMode.OFF:
+            # handle modules created in the fake filesystem
+            module_path = self.fake_module_path(fullname)
+            if module_path:
+                spec = spec_from_file_location(fullname, module_path)
+                if spec:
+                    module = module_from_spec(spec)
+                    sys.modules[fullname] = module
+                    return ModuleSpec(fullname, self)
         return None
 
     def load_module(self, fullname: str) -> ModuleType:
